package database

import (
	"fmt"
	"log"

	"git.paulbsd.com/paulbsd/qrz/src/config"
	"git.paulbsd.com/paulbsd/qrz/src/qrz"
	_ "github.com/lib/pq"
	"xorm.io/xorm"
	"xorm.io/xorm/names"
)

// Initialize creates connection to database and exec Schema
func Initialize(config *config.Config) (err error) {
	var connstr string
	switch config.DbType {
	case "mysql":
		connstr = fmt.Sprintf("%s:%s@%s/%s",
			config.DbUsername,
			config.DbPassword,
			config.DbHostname,
			config.DbName)
	default:
		// postgresql by default
		connstr = fmt.Sprintf("%s://%s:%s@%s/%s",
			config.DbType,
			config.DbUsername,
			config.DbPassword,
			config.DbHostname,
			config.DbName)
	}
	config.Db, err = xorm.NewEngine(config.DbType, connstr)
	if err != nil {
		log.Fatalln(err)
	}

	config.Db.SetMapper(names.GonicMapper{})

	err = config.Db.CreateTables(qrz.Qrz{})
	if err != nil {
		log.Fatalln(err)
	}

	err = config.Db.Sync2(qrz.Qrz{})
	if err != nil {
		log.Fatalln(err)
	}

	config.Db.ShowSQL(config.Debug)

	return
}
