//go:generate packr2 -v

package templates

import (
	"embed"
	"fmt"
	"html/template"
	"io"
	"log"
	"os"
	"strings"

	"github.com/labstack/echo/v4"
	"github.com/markbates/pkger"
)

// Render is a method that render templates
func (t *Template) Render(w io.Writer, name string, data interface{}, c echo.Context) (err error) {
	if strings.HasSuffix(name, ".html") {
		c.Response().Header().Set(echo.HeaderContentType, echo.MIMETextHTMLCharsetUTF8)
	}
	return t.templates.ExecuteTemplate(w, name, data)
}

// BuildTemplates converts packr packages to html/template
func BuildTemplates(fs *embed.FS) (builttemplates *Template, err error) {
	basedir := "templates"
	tmpl := template.New(basedir)

	dir, err := fs.ReadDir(basedir)
	if err != nil {
		log.Println(err)
	}
	for _, filename := range dir {
		tmplContent, err := fs.ReadFile(fmt.Sprintf("%s/%s", basedir, filename.Name()))
		if err != nil {
			log.Println(err)
		}
		tmpl.New(filename.Name()).Parse(string(tmplContent))
	}

	builttemplate := &Template{
		templates: tmpl,
	}

	return builttemplate, nil
}

// BuildTemplatesDir converts packr packages to html/template
func BuildTemplatesDir(dir string) (builttemplates *Template, err error) {
	tmpl := template.New("templates")

	err = pkger.Walk(dir, func(path string, info os.FileInfo, err error) error {
		if err != nil {
			return err
		}
		freader, err := pkger.Open(path)
		if err != nil {
			log.Println(err)
		}
		tmplContent, err := io.ReadAll(freader)
		tmpl.New(info.Name()).Parse(string(tmplContent))
		log.Println(info.Name(), tmplContent)
		return err
	})
	if err != nil {
		log.Println(err)
	}

	builttemplate := &Template{
		templates: tmpl,
	}

	return builttemplate, nil
}

// Template is a template struct
type Template struct {
	templates *template.Template
}
